/**
 * Copyright (c) 2016-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.extension.command.monitoring.so.api;

import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.jmx.JMXPetalsAdministration;
import org.ow2.petals.cli.shell.command.AbstractPetalsCliCommand;
import org.ow2.petals.cli.shell.util.PetalsAdminWrapper;
import org.ow2.petals.jmx.api.api.JMXClient;

/**
 * TODO see {@link AbstractPetalsCliCommand}
 *
 */
public abstract class AbstractSubscriptionObject implements SubscriptionObject {

    private PetalsAdminWrapper adminWrapper;

    private JMXClient jmxClient;

    protected PetalsAdminWrapper getAdminWrapper() {
        if (adminWrapper == null) {
            if (getShell() == null) {
                throw new IllegalStateException(
                        "This can't be called in the constructor since the shell is set after construction");
            }
            this.adminWrapper = new PetalsAdminWrapper(getShell());
        }
        return adminWrapper;
    }

    protected PetalsAdministration getAdmin() {
        return getAdminWrapper().getAdmin();
    }

    /**
     * Use ONLY if JMXClient itself is needed, and do not forget to inject it in {@link #jmxClient} in tests!
     * 
     * @return a lower-level JMXClient
     */
    public JMXClient getJMXClient() {
        // client must absolutely be injected for the tests!
        if (jmxClient == null) {
            // we must cast because PetalsAdministration does not support the functionalities provided by it
            jmxClient = ((JMXPetalsAdministration) getAdmin()).getJMXClient();
        }
        return jmxClient;
    }

    /**
     * This must be public to be accessible from tests
     */
    public void setJMXClient(JMXClient jmxClient) {
        this.jmxClient = jmxClient;
    }
}
